import React, { PropsWithChildren } from "react";
import { IconProps } from "../icons";
import { TestProps } from "../../util";
export type CollapsibleComponentRef = {
    /**
     * Function that triggers the collapsed component to open. If it's open already,
     * it won't do anything.
     */
    open: () => void;
    /**
     * Function that triggers the opened component to collapse. If it's collapsed already,
     * it won't do anything.
     */
    collapse: () => void;
};
export type CollapsibleComponentProps = TestProps & {
    /**
     * CollapsibleComponent will render the result of this function.
     * @param CollapsibleContent  A component that renders its children wrapped in an animated view.
     * @param CollapseIcon  An animated icon that initiates the collapsing/expanding of CollapsibleContent.
     * @param toggle  A function that initiates the collapsing/expanding of CollapsibleContent. This enables toggling by other means than using CollapseIcon, e.g. you can pass this function to a Button.
     */
    render: (CollapsibleContent: React.ComponentType<PropsWithChildren>, CollapseIcon: React.ComponentType<CollapseIconProps>, toggle: () => void) => React.ReactElement<any, any>;
    /**
     * Whether to show the collapsible content on initial render.
     *
     * Defaults to `false`.
     */
    openInitially?: boolean;
    /**
     * Whether the container should be growing upwards instead of downwards when open.
     * Controls the initial orientation of the DownIcon.
     *
     * Defaults to `false`.
     */
    growUpwards?: boolean;
};
export type CollapseIconProps = IconProps & {
    /**
     * Whether icon can be pressed.
     * Useful for disabling pressable when something else handles the press
     * (e.g. the entire list item) and the arrow is only used as visual feedback.
     *
     * Defaults to `true`.
     */
    pressable?: boolean;
};
/**
 * A component that handles the collapse animation and gives us
 * an easy way to create custom collapsible components.
 */
export declare const CollapsibleComponent: React.ForwardRefExoticComponent<TestProps & {
    /**
     * CollapsibleComponent will render the result of this function.
     * @param CollapsibleContent  A component that renders its children wrapped in an animated view.
     * @param CollapseIcon  An animated icon that initiates the collapsing/expanding of CollapsibleContent.
     * @param toggle  A function that initiates the collapsing/expanding of CollapsibleContent. This enables toggling by other means than using CollapseIcon, e.g. you can pass this function to a Button.
     */
    render: (CollapsibleContent: React.ComponentType<PropsWithChildren>, CollapseIcon: React.ComponentType<CollapseIconProps>, toggle: () => void) => React.ReactElement<any, any>;
    /**
     * Whether to show the collapsible content on initial render.
     *
     * Defaults to `false`.
     */
    openInitially?: boolean | undefined;
    /**
     * Whether the container should be growing upwards instead of downwards when open.
     * Controls the initial orientation of the DownIcon.
     *
     * Defaults to `false`.
     */
    growUpwards?: boolean | undefined;
} & React.RefAttributes<CollapsibleComponentRef>>;
//# sourceMappingURL=collapsible-component.d.ts.map