import type { MetaDocument, Country, MetaActivityDocument } from "./types";
export declare type RNPrivacyComponentType = {
    /**
     * Initializes the underlying privacy provider instance. Must be called before accessing any of the privacy policy related functions!
     * @param leanIX - project's lean id
     * @param productionMode - use the production API or the preview
     */
    initPrivacyProvider(leanIX: string, productionMode: boolean): void;
    /**
     * Accepts a given privacy policy document.
     * @param documentId - the document's id
     */
    acceptPrivacyPolicy(documentId: string): void;
    /**
     * Checks the status of a privacy policy document.
     * @param documentId - the document's id
     * @returns Promise with a boolean value
     */
    isPrivacyPolicyAccepted(documentId: string): Promise<boolean>;
    /**
     * Retrieves the available privacy policy meta documents.
     * @returns - Promise object with a MetaDocument array
     */
    getAvailablePrivacyPolicyDocuments(): Promise<MetaDocument[]>;
    /**
     * Returns with the timestamp of the last online check of a privacy policy document.
     * @param country - country code
     * @param language - language code
     * @returns Promise object with a Date as string (e.g.: Android: Thu May 30 12:18:16 GMT+03:00 2019, iOS: 2019-05-29T10:53:23)
     */
    getPrivacyPolicyLastCheckedTimestamp(country: string, language: string): Promise<string>;
    /**
     * Retrieves the accept date of a privacy policy document.
     * @param documentId - the document's id
     * @returns Promise object with a Date as string (e.g.: Android: Thu May 30 12:18:16 GMT+03:00 2019, iOS: 2019-05-29T10:53:23)
     */
    getTimestampOfPrivacyPolicyConsent(documentId: string): Promise<string>;
    /**
     * Returns a document based on country and language.
     * @param country - country code
     * @param language - language code
     * @returns Promise object containing the requested document
     */
    getPrivacyPolicyDocument(country: string, language: string): Promise<MetaDocument>;
    /**
     * Returns the common countries from the online and offline countries. "Online countries" means those that are available on the privacy portal.
     * "Offline countries" means that the app stores the privacy texts for this countries by default.
     * @param countriesFromApp - list of the supported offline countries
     * @returns Promise object with the common countries
     */
    getAvailablePrivacyPolicyCountries(countriesFromApp: Country[]): Promise<Country[]>;
    /**
     * Initializes the underlying terms of use provider instance. Must be called before accessing any of the terms of use related functions!
     */
    initTermsOfUseProvider(): void;
    /**
     * Accepts a given terms of use document.
     * @param documentId - the document's id
     */
    acceptTermsOfUse(documentId: string): void;
    /**
     * Checks the status of a terms of use document.
     * @param documentId - the document's id
     * @returns Promise with a boolean value
     */
    isTermsOfUseAccepted(documentId: string): Promise<boolean>;
    /**
     * Returns with the timestamp of the last online check of a terms of use document.
     * @param country - country code
     * @param language - language code
     * @returns Promise object with a Date as string (e.g.: Android: Thu May 30 12:18:16 GMT+03:00 2019, iOS: 2019-05-29T10:53:23)
     */
    getTermsOfUseLastCheckedTimestamp(country: string, language: string): Promise<string>;
    /**
     * Retrieves the accept date of a terms of use document.
     * @param documentId - the document's id
     * @returns Promise object with a Date as string (e.g.: Android: Thu May 30 12:18:16 GMT+03:00 2019, iOS: 2019-05-29T10:53:23)
     */
    getTimestampOfTermsOfUseConsent(documentId: string): Promise<string>;
    /**
     * Retrieves the available terms of use meta documents.
     * @returns - Promise object with a MetaDocument array
     */
    getAvailableTermsOfUseDocuments(): Promise<MetaDocument[]>;
    /**
     * Returns a terms of use document based on country and language.
     * @param country - country code
     * @param language - language code
     * @returns Promise object containing the requested document
     */
    getTermsOfUseDocument(country: string, language: string): Promise<MetaDocument>;
    /**
     * Returns the common countries from the app and offline countries.
     * "Offline countries" means that the app stores the terms of use documents for these countries by default.
     * @param countriesFromApp - list of the supported offline countries
     * @returns Promise object with the common countries
     */
    getAvailableTermsOfUseCountries(countriesFromApp: Country[]): Promise<Country[]>;
    /**
     * Initializes the underlying activity consent provider instance. Must be called before accessing any of the activity consent related functions!
     * @param activityId - project's activity id
     */
    initActivityConsentProvider(activityId: string): void;
    /**
     * Accepts a given activity consent document.
     * @param documentId - the document's id
     */
    acceptActivityConsent(documentId: string): void;
    /**
     * Checks the status of an activity consent document.
     * @param documentId - the document's id
     * @returns Promise with a boolean value
     */
    isActivityConsentAccepted(documentId: string): Promise<boolean>;
    /**
     * Returns with the timestamp of the last online check of an activity consent document.
     * @param country - country code
     * @param language - language code
     * @returns Promise object with a Date as string (e.g.: Android: Thu May 30 12:18:16 GMT+03:00 2019, iOS: 2019-05-29T10:53:23)
     */
    getActivityConsentLastCheckedTimestamp(country: string, language: string): Promise<string>;
    /**
     * Retrieves the accept date of an activity consent document.
     * @param documentId - the document's id
     * @returns Promise object with a Date as string (e.g.: Android: Thu May 30 12:18:16 GMT+03:00 2019, iOS: 2019-05-29T10:53:23)
     */
    getTimestampOfActivityConsent(documentId: string): Promise<string>;
    /**
     * Returns the available activity consent documents.
     * @returns - Promise object with a `MetaActivityDocument` array
     */
    getAvailableActivityConsentDocuments(): Promise<MetaActivityDocument[]>;
    /**
     * Returns an activity consent document based on country and language.
     * @param country - country code
     * @param language - language code
     */
    getActivityConsentDocument(country: string, language: string): Promise<MetaActivityDocument>;
    /**
     * Returns the common countries from the online and offline countries. "Online countries" means those that are available on the auth toolkit.
     * "Offline countries" means that the app stores the activity consent documents for these countries by default.
     * @param countriesFromApp - list of the supported offline countries
     * @returns Promise object with the common countries
     */
    getAvailableActivityConsentCountries(countriesFromApp: Country[]): Promise<Country[]>;
};
export declare const RNPrivacyComponent: RNPrivacyComponentType;
//# sourceMappingURL=native-modules.d.ts.map